import 'package:json_api_document/json_api_document.dart';
import 'package:json_matcher/json_matcher.dart';
import 'package:test/test.dart';

main() {
  test('Resource may have self link', () {
    final res = Resource('apples', '1',
        attributes: {'color': 'red'}, self: Link('/apples/1'));
    expect(
        res,
        encodesToJson({
          "type": "apples",
          "id": "1",
          "attributes": {"color": "red"},
          "links": {"self": "/apples/1"}
        }));
  });

  test('Resource may have relationships', () {
    final res = Resource('articles', '1',
        attributes: {'title': 'JSON API paints my bikeshed!'},
        self: Link('http://example.com/articles/1'),
        relationships: {
          'author': Relationship.toOne(ResourceIdentifier('people', '9'),
              self: Link('http://example.com/articles/1/relationships/author'),
              related: Link('http://example.com/articles/1/author')),
          'comments': Relationship.toMany([
            ResourceIdentifier('comments', '5'),
            ResourceIdentifier('comments', '12')
          ],
              self:
                  Link('http://example.com/articles/1/relationships/comments'),
              related: Link('http://example.com/articles/1/comments')),
        });

    expect(
        res,
        encodesToJson({
          "type": "articles",
          "id": "1",
          "attributes": {"title": "JSON API paints my bikeshed!"},
          "links": {"self": "http://example.com/articles/1"},
          "relationships": {
            "author": {
              "links": {
                "self": "http://example.com/articles/1/relationships/author",
                "related": "http://example.com/articles/1/author"
              },
              "data": {"type": "people", "id": "9"}
            },
            "comments": {
              "links": {
                "self": "http://example.com/articles/1/relationships/comments",
                "related": "http://example.com/articles/1/comments"
              },
              "data": [
                {"type": "comments", "id": "5"},
                {"type": "comments", "id": "12"}
              ]
            }
          }
        }));
  });
}
